// ----------------------------------------------------------------------------
//
// VDream Component Suite version 8.0
//
// http://www.gilgil.net
//
// Copyright (c) Gilbert Lee All rights reserved
//
// ----------------------------------------------------------------------------

import java.util.ArrayList;
import java.util.HashMap;

// ----------------------------------------------------------------------------
// PerformanceVerbose
// ----------------------------------------------------------------------------
class PerformanceVerbose
{
	public int  begMilestone;
	public int  endMilestone;
	public long duration;	
}

//----------------------------------------------------------------------------
// PerformanceVerboseList
//----------------------------------------------------------------------------
class PerformanceVerboseList extends ArrayList<PerformanceVerbose>
{
	private static final long serialVersionUID = 1L;
}

//----------------------------------------------------------------------------
// PerformanceReportKey
//----------------------------------------------------------------------------
class PerformanceReportKey
{
	public int begMilestone;
	public int endMileseone;
	
	public int hashCode()
	{
	   return begMilestone + endMileseone;
	}
	
	public boolean equals(Object obj)
	{
		if (obj == null) return false;
		if (obj == this) return true;
		if (!(obj instanceof PerformanceReportKey)) return false;
		
		PerformanceReportKey rhs = (PerformanceReportKey)obj;
		if (this.begMilestone != rhs.begMilestone) return false;
		if (this.endMileseone != rhs.endMileseone) return false;
		
		return true;
	}
}

//----------------------------------------------------------------------------
// PerformanceReportData
//----------------------------------------------------------------------------
class PerformanceReportData
{
	public int  count;
	public long totalDuration;
}

//----------------------------------------------------------------------------
// PerformanceReportMap
//----------------------------------------------------------------------------
class PerformanceReportMap extends HashMap<PerformanceReportKey, PerformanceReportData>
{
	private static final long serialVersionUID = 1L;
}

//----------------------------------------------------------------------------
// Performance
//----------------------------------------------------------------------------
public class Performance
{
	public boolean verbose;
	
	public PerformanceVerboseList  verboseList;
	public PerformanceReportMap    reportMap;
	
	protected int                  lastMilestone;
	protected long                 lastTick;	
	
	public Performance()
	{
		verbose     = false;
		verboseList = new PerformanceVerboseList();
		reportMap   = new PerformanceReportMap();
		clear();
	}
		
	public void report()
	{
		if (verbose)
		{
			System.out.println("beg\tend\tduration");
			for (PerformanceVerbose verbose : verboseList)
			{
				if (verbose.begMilestone == 0) continue;
				System.out.println(String.format("%d\t%d\t%d", verbose.begMilestone, verbose.endMilestone, verbose.duration));
			} 
		} else
		{
			System.out.println("beg\tend\tcount\tduration");
			
			for (PerformanceReportKey key : reportMap.keySet())
			{
				if (key.begMilestone == 0) continue;
				PerformanceReportData data = reportMap.get(key);
				System.out.println(String.format("%d\t%d\t%d\t%d", key.begMilestone, key.endMileseone, data.count, data.totalDuration));
			}
		}
	}
	
	public void clear()
	{
		verboseList.clear();
		reportMap.clear();
		lastMilestone = 0;
		lastTick      = System.nanoTime();
	}
	
	public void check(int milestone)
	{
		check(milestone, System.nanoTime());
	}
	
	public void check(int milestone, long now)
	{
		if (verbose)
		{
			PerformanceVerbose verbose = new PerformanceVerbose();
			verbose.begMilestone = lastMilestone;
			verbose.endMilestone = milestone;
			verbose.duration     = now - lastTick;
			verboseList.add(verbose);
		} else
		{
			PerformanceReportKey key = new PerformanceReportKey();
			key.begMilestone = lastMilestone;
			key.endMileseone = milestone;

			PerformanceReportData data = reportMap.get(key);
			if (data == null)
			{
				data = new PerformanceReportData();
				reportMap.put(key, data);
			}
			data.count++;
			data.totalDuration += now - lastTick;
		}
		lastMilestone = milestone;
		lastTick      = now;
	}
}
